@@ -1,9 +1,8 @@
 # -*- coding: utf-8 -
 #
-# This file is part of gunicorn released under the MIT license. 
+# This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 
-import logging
 import os
 import pkg_resources
 import sys
@@ -21,7 +20,7 @@ class PasterBaseApplication(Application):
         cx = loadwsgi.loadcontext(SERVER, self.cfgurl, relative_to=self.relpath)
         gc, lc = cx.global_conf.copy(), cx.local_conf.copy()
         cfg = {}
-        
+
         host, port = lc.pop('host', ''), lc.pop('port', '')
         if host and port:
             cfg['bind'] = '%s:%s' % (host, port)
@@ -44,10 +43,11 @@ class PasterBaseApplication(Application):
 
         return cfg
 
-    def configure_logging(self):
+    def load_config(self):
+        super(PasterBaseApplication, self).load_config()
+
+        # reload logging conf
         if hasattr(self, "cfgfname"):
-            self.logger = logging.getLogger('gunicorn')
-            # from paste.script.command
             parser = ConfigParser.ConfigParser()
             parser.read([self.cfgfname])
             if parser.has_section('loggers'):
@@ -61,11 +61,11 @@ class PasterBaseApplication(Application):
                 config_file = os.path.abspath(self.cfgfname)
                 fileConfig(config_file, dict(__file__=config_file,
                                              here=os.path.dirname(config_file)))
-                return
-        super(PasterBaseApplication, self).configure_logging()
+
+
 
 class PasterApplication(PasterBaseApplication):
-    
+
     def init(self, parser, opts, args):
         if len(args) != 1:
             parser.error("No application name specified.")
@@ -81,14 +81,14 @@ class PasterApplication(PasterBaseApplication):
 
         sys.path.insert(0, self.relpath)
         pkg_resources.working_set.add_entry(self.relpath)
-        
+
         return self.app_config()
-        
+
     def load(self):
         return loadapp(self.cfgurl, relative_to=self.relpath)
-    
+
 class PasterServerApplication(PasterBaseApplication):
-    
+
     def __init__(self, app, gcfg=None, host="127.0.0.1", port=None, *args, **kwargs):
         self.cfg = Config()
         self.app = app
@@ -110,12 +110,12 @@ class PasterServerApplication(PasterBaseApplication):
         cfg["bind"] = bind
 
         if gcfg:
-            for k, v in list(gcfg.items()):
+            for k, v in gcfg.items():
                 cfg[k] = v
             cfg["default_proc_name"] = cfg['__file__']
 
         try:
-            for k, v in list(cfg.items()):
+            for k, v in cfg.items():
                 if k.lower() in self.cfg.settings and v is not None:
                     self.cfg.set(k.lower(), v)
         except Exception, e:
@@ -123,12 +123,11 @@ class PasterServerApplication(PasterBaseApplication):
             sys.stderr.flush()
             sys.exit(1)
 
-        self.configure_logging()
 
     def load_config(self):
         if not hasattr(self, "cfgfname"):
             return
-        
+
         cfg = self.app_config()
         for k,v in cfg.items():
             try:
@@ -138,7 +137,7 @@ class PasterServerApplication(PasterBaseApplication):
                 raise
 
     def load(self):
-        if hasattr(self, "cfgfname"):
+        if self.app is None and hasattr(self, "cfgfname"):
             return loadapp(self.cfgurl, relative_to=self.relpath)
         return self.app
 
@@ -154,14 +153,14 @@ def run():
 def paste_server(app, gcfg=None, host="127.0.0.1", port=None, *args, **kwargs):
     """\
     A paster server.
-    
+
     Then entry point in your paster ini file should looks like this:
-    
+
     [server:main]
     use = egg:gunicorn#main
     host = 127.0.0.1
     port = 5000
-    
+
     """
     from gunicorn.app.pasterapp import PasterServerApplication
     PasterServerApplication(app, gcfg=gcfg, host=host, port=port, *args, **kwargs).run()
