@@ -212,6 +212,13 @@ def validate_string(val):
         raise TypeError("Not a string: %s" % val)
     return val.strip()
 
+def validate_class(val):
+    if inspect.isfunction(val) or inspect.ismethod(val):
+        val = val()
+    if inspect.isclass(val):
+        return val
+    return validate_string(val)
+
 def validate_callable(arity):
     def _validate_callable(val):
         if not callable(val):
@@ -266,7 +273,7 @@ def validate_post_request(val):
     elif largs == 2:
         return wrap_post_request(val)
     else:
-         raise TypeError("Value must have an arity of: 3")
+        raise TypeError("Value must have an arity of: 3")
 
 
 
@@ -338,7 +345,7 @@ class WorkerClass(Setting):
     section = "Worker Processes"
     cli = ["-k", "--worker-class"]
     meta = "STRING"
-    validator = validate_string
+    validator = validate_class
     default = "sync"
     desc = """\
         The type of workers to use.
@@ -474,7 +481,7 @@ class LimitRequestFieldSize(Setting):
     type = "int"
     default = 8190
     desc= """\
-        Limit he allowed size of an HTTP request header field.
+        Limit the allowed size of an HTTP request header field.
 
         Value is a number from 0 (unlimited) to 8190. to set the limit
         on the allowed size of an HTTP request header field.
@@ -669,7 +676,7 @@ class AccessLog(Setting):
     desc = """\
         The Access log file to write to.
 
-        "-" means log to stdout.
+        "-" means log to stderr.
         """
 
 class AccessLogFormat(Setting):
@@ -713,7 +720,7 @@ class ErrorLog(Setting):
     desc = """\
         The Error log file to write to.
 
-        "-" means log to stdout.
+        "-" means log to stderr.
         """
 
 class Loglevel(Setting):
@@ -740,7 +747,7 @@ class LoggerClass(Setting):
     section = "Logging"
     cli = ["--logger-class"]
     meta = "STRING"
-    validator = validate_string
+    validator = validate_class
     default = "simple"
     desc = """\
         The logger you want to use to log events in gunicorn.
