@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -
 #
-# This file is part of gunicorn released under the MIT license. 
+# This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 
 from __future__ import with_statement
@@ -21,13 +21,14 @@ class AsyncWorker(base.Worker):
     def __init__(self, *args, **kwargs):
         super(AsyncWorker, self).__init__(*args, **kwargs)
         self.worker_connections = self.cfg.worker_connections
-    
+
     def timeout_ctx(self):
         raise NotImplementedError()
 
     def handle(self, client, addr):
+        req = None
         try:
-            parser = http.RequestParser(client)
+            parser = http.RequestParser(self.cfg, client)
             try:
                 while True:
                     req = None
@@ -36,8 +37,10 @@ class AsyncWorker(base.Worker):
                     if not req:
                         break
                     self.handle_request(req, client, addr)
-            except StopIteration:
-                pass
+            except StopIteration, e:
+                self.log.debug("Closing connection. %s", e)
+            except Exception, e:
+                self.handle_error(req, client, addr, e)
         except socket.error, e:
             if e[0] not in (errno.EPIPE, errno.ECONNRESET):
                 self.log.exception("Socket error processing request.")
@@ -47,14 +50,14 @@ class AsyncWorker(base.Worker):
                 else:
                     self.log.debug("Ignoring EPIPE")
         except Exception, e:
-            self.handle_error(client, e)
+            self.handle_error(req, client, addr, e)
         finally:
             util.close(client)
 
     def handle_request(self, req, sock, addr):
+        request_start = datetime.now()
         try:
             self.cfg.pre_request(self, req)
-            request_start = datetime.now()
             resp, environ = wsgi.create(req, sock, addr, self.address, self.cfg)
             self.nr += 1
             if self.alive and self.nr >= self.max_requests:
@@ -68,8 +71,8 @@ class AsyncWorker(base.Worker):
                 for item in respiter:
                     resp.write(item)
                 resp.close()
-                request_time = request_start - datetime.now()
-                self.log.access(resp, environ, request_time)
+                request_time = datetime.now() - request_start
+                self.log.access(resp, req, environ, request_time)
             finally:
                 if hasattr(respiter, "close"):
                   respiter.close()
