@@ -1,12 +1,10 @@
 # -*- coding: utf-8 -
 #
-# This file is part of gunicorn released under the MIT license. 
+# This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 
-
-import logging
+from datetime import datetime
 import os
-import random
 import signal
 import sys
 import traceback
@@ -14,10 +12,10 @@ import traceback
 
 from gunicorn import util
 from gunicorn.workers.workertmp import WorkerTmp
-
 from gunicorn.http.errors import InvalidHeader, InvalidHeaderName, \
-InvalidRequestLine, InvalidRequestMethod, InvalidHTTPVersion
-
+InvalidRequestLine, InvalidRequestMethod, InvalidHTTPVersion, \
+LimitRequestLine, LimitRequestHeaders
+from gunicorn.http.wsgi import default_environ, Response
 
 class Worker(object):
 
@@ -25,10 +23,10 @@ class Worker(object):
         lambda x: getattr(signal, "SIG%s" % x),
         "HUP QUIT INT TERM USR1 USR2 WINCH CHLD".split()
     )
-    
+
     PIPE = []
 
-    def __init__(self, age, ppid, socket, app, timeout, cfg):
+    def __init__(self, age, ppid, socket, app, timeout, cfg, log):
         """\
         This is called pre-fork so it shouldn't do anything to the
         current process. If there's a need to make process wide
@@ -45,14 +43,14 @@ class Worker(object):
         self.nr = 0
         self.max_requests = cfg.max_requests or sys.maxint
         self.alive = True
-        self.log = logging.getLogger(__name__)
+        self.log = log
         self.debug = cfg.debug
         self.address = self.socket.getsockname()
-        self.tmp = WorkerTmp(cfg) 
-        
+        self.tmp = WorkerTmp(cfg)
+
     def __str__(self):
         return "<Worker %s>" % self.pid
-        
+
     @property
     def pid(self):
         return os.getpid()
@@ -83,20 +81,23 @@ class Worker(object):
         util.set_owner_process(self.cfg.uid, self.cfg.gid)
 
         # Reseed the random number generator
-        random.seed()
+        util.seed()
 
         # For waking ourselves up
         self.PIPE = os.pipe()
         map(util.set_non_blocking, self.PIPE)
         map(util.close_on_exec, self.PIPE)
-        
+
         # Prevent fd inherientence
         util.close_on_exec(self.socket)
         util.close_on_exec(self.tmp.fileno())
+
+        self.log.close_on_exec()
+
         self.init_signals()
-        
+
         self.wsgi = self.app.wsgi()
-        
+
         # Enter main run loop
         self.booted = True
         self.run()
@@ -107,7 +108,16 @@ class Worker(object):
         signal.signal(signal.SIGTERM, self.handle_exit)
         signal.signal(signal.SIGINT, self.handle_exit)
         signal.signal(signal.SIGWINCH, self.handle_winch)
-            
+        signal.signal(signal.SIGUSR1, self.handle_usr1)
+        # Don't let SIGQUIT and SIGUSR1 disturb active requests
+        # by interrupting system calls
+        if hasattr(signal, 'siginterrupt'):  # python >= 2.6
+            signal.siginterrupt(signal.SIGQUIT, False)
+            signal.siginterrupt(signal.SIGUSR1, False)
+
+    def handle_usr1(self, sig, frame):
+        self.log.reopen_files()
+
     def handle_quit(self, sig, frame):
         self.alive = False
 
@@ -115,35 +125,58 @@ class Worker(object):
         self.alive = False
         sys.exit(0)
 
-    def handle_error(self, client, exc):
+    def handle_error(self, req, client, addr, exc):
+        request_start = datetime.now()
         if isinstance(exc, (InvalidRequestLine, InvalidRequestMethod,
             InvalidHTTPVersion, InvalidHeader, InvalidHeaderName,)):
-            
+
+            status_int = 400
+            reason = "Bad Request"
+
             if isinstance(exc, InvalidRequestLine):
                 mesg = "<p>Invalid Request Line '%s'</p>" % str(exc)
             elif isinstance(exc, InvalidRequestMethod):
-                mesg = "<p>Invalid Method'%s'</p>" % str(exc)
+                mesg = "<p>Invalid Method '%s'</p>" % str(exc)
             elif isinstance(exc, InvalidHTTPVersion):
                 mesg = "<p>Invalid HTTP Version '%s'</p>" % str(exc)
             elif isinstance(exc, (InvalidHeaderName, InvalidHeader,)):
-                mesg = "<p>Invalid Header'%s'</p>" % str(exc)
-            reason = "Bad Request"
-            status_int = 400
+                mesg = "<p>Invalid Header '%s'</p>" % str(exc)
+            elif isinstance(exc, LimitRequestLine):
+                mesg = "<p>%s</p>" % str(exc)
+            elif isinstance(exc, LimitRequestHeaders):
+                mesg = "<p>Error parsing headers: '%s'</p>" % str(exc)
+
+            self.log.debug("Invalid request from ip={ip}: {error}"\
+                           "".format(ip=client.getpeername()[0],
+                                     error=repr(exc),
+                                    )
+                          )
         else:
-            mesg = reason = "Internal Server reason"
+            self.log.exception("Error handling request")
+
             status_int = 500
-            
+            reason = "Internal Server Error"
+            mesg = ""
+
+        if req is not None:
+            request_time = datetime.now() - request_start
+            environ = default_environ(req, client, self.cfg)
+            environ['REMOTE_ADDR'] = addr[0]
+            environ['REMOTE_PORT'] = str(addr[1])
+            resp = Response(req, client)
+            resp.status = "%s %s" % (status_int, reason)
+            resp.response_length = len(mesg)
+            self.log.access(resp, req, environ, request_time)
+
         if self.debug:
             tb =  traceback.format_exc()
-            mesg += "<h2>Traceback:</23><pre>%s</pre>" % tb
+            mesg += "<h2>Traceback:</h2>\n<pre>%s</pre>" % tb
 
         try:
-            util.write_error(client, mesg, status_int=status_int, 
-                    reason=reason)
+            util.write_error(client, status_int, reason, mesg)
         except:
-            if self.debug:
-                self.log.warning("Unexpected error %s" % traceback.format_exc())
-        
+            self.log.warning("Failed to send error message.")
+
     def handle_winch(self, sig, fname):
         # Ignore SIGWINCH in worker. Fixes a crash on OpenBSD.
         return
