@@ -87,7 +87,7 @@ class GeventWorker(AsyncWorker):
         except gevent.GreenletExit:
             pass
 
-    if hasattr(gevent.core, 'dns_shutdown'):
+    if gevent.version_info[0] == 0:
 
         def init_process(self):
             #gevent 0.13 and older doesn't reinitialize dns for us after forking
@@ -96,13 +96,29 @@ class GeventWorker(AsyncWorker):
             gevent.core.dns_init()
             super(GeventWorker, self).init_process()
 
+
+class GeventResponse(object):
+
+    status = None
+    headers = None
+    response_length = None
+
+
+    def __init__(self, status, headers, clength):
+        self.status = status
+        self.headers = headers
+        self.response_length = clength
+
 class PyWSGIHandler(pywsgi.WSGIHandler):
 
     def log_request(self):
         start = datetime.fromtimestamp(self.time_start)
         finish = datetime.fromtimestamp(self.time_finish)
         response_time = finish - start
-        self.server.log.access(self, self.environ, response_time)
+        resp = GeventResponse(self.status, self.response_headers,
+                self.response_length)
+        req_headers = [h.split(":", 1) for h in self.headers.headers]
+        self.server.log.access(resp, req_headers, self.environ, response_time)
 
     def get_environ(self):
         env = super(PyWSGIHandler, self).get_environ()
