@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -
 #
-# This file is part of gunicorn released under the MIT license. 
+# This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 #
 
@@ -16,7 +16,7 @@ import gunicorn.util as util
 import gunicorn.workers.base as base
 
 class SyncWorker(base.Worker):
-    
+
     def run(self):
         # self.socket appears to lose its blocking status after
         # we fork in the arbiter. Reset it here.
@@ -24,7 +24,7 @@ class SyncWorker(base.Worker):
 
         while self.alive:
             self.notify()
-            
+
             # Accept a connection. If we get an error telling us
             # that no connection is waiting we fall down to the
             # select which is where we'll wait for a bit for new
@@ -48,7 +48,7 @@ class SyncWorker(base.Worker):
             if self.ppid != os.getppid():
                 self.log.info("Parent changed, shutting down: %s", self)
                 return
-            
+
             try:
                 self.notify()
                 ret = select.select([self.socket], [], self.PIPE, self.timeout)
@@ -63,22 +63,23 @@ class SyncWorker(base.Worker):
                     else:
                         return
                 raise
-    
+
     def handle(self, client, addr):
+        req = None
         try:
-            parser = http.RequestParser(client)
+            parser = http.RequestParser(self.cfg, client)
             req = parser.next()
             self.handle_request(req, client, addr)
-        except StopIteration:
-            self.log.debug("Ignored premature client disconnection.")
+        except StopIteration, e:
+            self.log.debug("Closing connection. %s", e)
         except socket.error, e:
             if e[0] != errno.EPIPE:
                 self.log.exception("Error processing request.")
             else:
                 self.log.debug("Ignoring EPIPE")
         except Exception, e:
-            self.handle_error(client, e)
-        finally:    
+            self.handle_error(req, client, addr, e)
+        finally:
             util.close(client)
 
     def handle_request(self, req, client, addr):
@@ -105,7 +106,7 @@ class SyncWorker(base.Worker):
                         resp.write(item)
                 resp.close()
                 request_time = datetime.now() - request_start
-                self.log.access(resp, environ, request_time)
+                self.log.access(resp, req, environ, request_time)
             finally:
                 if hasattr(respiter, "close"):
                     respiter.close()
@@ -113,7 +114,7 @@ class SyncWorker(base.Worker):
             raise
         except Exception, e:
             # Only send back traceback in HTTP in debug mode.
-            self.handle_error(client, e) 
+            self.handle_error(req, client, addr, e)
             return
         finally:
             try:
